import bpy
import os
from bpy.types import Operator
import addon_utils
from . import rain_icons


class RAIN_OP_add_shader(Operator):
    """ Add Rain Shader """
    bl_idname = "bagarain.add_rain_shader"
    bl_label = 'Add Rain Shader'
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        o = context.object
        
        if len(bpy.context.selected_objects) > 0 and o.type == 'MESH' and len(o.material_slots) > 0:
                    return context.selected_objects


    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)

    
    def draw(self, context):
        obj = context.object
        layout = self.layout
        col = layout.column()
        col.template_icon(icon_value = rain_icons.previews["addrain_icon"].icon_id, scale=10)
        col.label(text="Select target material :")
        col.template_list("RAIN_UL_shader_list", "", obj, "material_slots", obj, "active_material_index")
        col.separator(factor = 1.5)
        col.label(text="- Compiling this shader can take a few seconds.")
        col.label(text="- A rain effect will be added to the selected material.")
        col.label(text="- You can easily remove it directly in the shader editor.")


    def execute(self, context):
        obj = context.object
        idx = obj.active_material_index
        mat = obj.material_slots[idx].material

        # material_slots = obj.material_slots
        # for slot in material_slots:
        #     mat = slot.material
        node_tree = mat.node_tree
        
# GET THE ACTIVE NODE OUTPUT IN THE MATERIAL
        node_output = None
        for node in node_tree.nodes:
            if node.type == "OUTPUT_MATERIAL" and node.is_active_output == True:
                node_output = node
                break
            if node.type == "GROUP" and node.label == "Baga Rain Generator":
                print("Rain generator already present.")
                return {"FINISHED"}
                
        # CONTINUE IF MAT OUTPUT IS PRESENT
        if node_output is not None:
        
# ADD THE NODE GROUP
            try:
                brg_nodes = bpy.data.node_groups["BRG_Rain_Shader_Generator"]
            except:
                Import_Nodes(self,context)
                brg_nodes = bpy.data.node_groups["BRG_Rain_Shader_Generator"]

            brg_nodes_group  = node_tree.nodes.new(type='ShaderNodeGroup')
            brg_nodes_group.name = "Baga Rain Generator"
            brg_nodes_group.label = "Baga Rain Generator"

            brg_nodes_group.node_tree = brg_nodes



# SETUP NEW LOCATION FOR THE NODE GROUP AND MAT OUTPUT
            brg_nodes_group.width = 200
            brg_nodes_group.location = node_output.location
            node_output.location.x = node_output.location.x + 250
            
            

# GET LINKS TO THE MATERIAL OUTPUT
            if len(node_output.inputs[0].links) > 0:
                surface_link = node_output.inputs[0].links[0]
            else:
                surface_link = None
            if len(node_output.inputs[2].links) > 0:
                displace_link = node_output.inputs[2].links[0]
            else:
                displace_link = None
            
            
# LINK THE NODE GROUP BETWEEN THE GROUP OUTPUT AND THE PREVIOUS NODE
            if surface_link is not None:
                preceding_node_surf = surface_link.from_node
                input = surface_link.to_socket
                output = surface_link.from_socket
                
                if preceding_node_surf.type == "BSDF_PRINCIPLED":
                    if len(preceding_node_surf.inputs[0].links) > 0:
                        bsdf_link = preceding_node_surf.inputs[0].links[0]
                        color_source = bsdf_link.from_socket
                                
                        node_tree.links.new(color_source, brg_nodes_group.inputs[1])
                    else:
                        brg_nodes_group.inputs[1].default_value = preceding_node_surf.inputs[0].default_value
                else:
                    brg_nodes_group.inputs[1].default_value = (0,0,0,1)
                    


                node_tree.links.new(output, brg_nodes_group.inputs[0])
                node_tree.links.remove(surface_link)
                node_tree.links.new(brg_nodes_group.outputs[0], input)

            if displace_link is not None:
                preceding_node_disp = displace_link.from_node
                input = displace_link.to_socket
                output = displace_link.from_socket

                node_tree.links.new(output, brg_nodes_group.inputs[12])
                node_tree.links.remove(displace_link)
                node_tree.links.new(brg_nodes_group.outputs[1], input)
            else:
                brg_nodes_group.inputs[13].default_value = 0
            

        return {"FINISHED"}


###################################################################################
# IMPORT NODE GROUP
###################################################################################
def Import_Nodes(self,context):

    for mod in addon_utils.modules():
        if mod.bl_info['name'] == "Baga Rain Generator":
            filepath = mod.__file__
            if bpy.app.version_string.startswith("3.3"):
                file_path = filepath.replace("__init__.py","Baga Rain Shader_3.3.blend")
            else : 
                file_path = filepath.replace("__init__.py","Baga Rain Shader_3.4.blend")
        else: pass
    inner_path = "NodeTree"
    nodes_name = "BRG_Rain_Shader_Generator"

    bpy.ops.wm.append(
        filepath = os.path.join(file_path, inner_path, nodes_name),
        directory = os.path.join(file_path, inner_path),
        filename=nodes_name
        )


###################################################################################
# DISPLAY WARNING
###################################################################################
def Warning(message = "", title = "Message Box", icon = 'INFO'):

    def draw(self, context):
        self.layout.label(text=message)
    bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)


class RAIN_UL_shader_list(bpy.types.UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname):
        ob = data
        slot = item
        ma = slot.material
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            if ma:
                layout.prop(ma, "name", text="", emboss=False, icon_value=icon)
            else:
                layout.label(text="", translate=False, icon_value=icon)
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon_value=icon)
